
// PiIoctl.h - Version 0.1a
// A parallel port driver for the PromICE WINDOWS-NT
// (C) 1997-99 Grammar Engine Inc.

// Device type           -- in the "User Defined" range."

#define PI_TYPE 40000

// The IOCTL function codes from 0x800 to 0xFFF are for customer use.

#define IOCTL_PI_PARAMETER \
    CTL_CODE( PI_TYPE, 0x900, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_PI_CONTROL \
    CTL_CODE( PI_TYPE, 0x910, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_PI_WRITE_CHAR \
    CTL_CODE( PI_TYPE, 0x920, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_PI_READ_CHAR \
    CTL_CODE( PI_TYPE, 0x930, METHOD_BUFFERED, FILE_READ_ACCESS )

#define IOCTL_PI_WRITE_BLOCK \
    CTL_CODE( PI_TYPE, 0x940, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_PI_READ_BLOCK \
    CTL_CODE( PI_TYPE, 0x950, METHOD_BUFFERED, FILE_READ_ACCESS )

#define IOCTL_PI_DO_BACK \
    CTL_CODE( PI_TYPE, 0x960, METHOD_BUFFERED, FILE_READ_ACCESS )

// End-O-File