/*	piconfig.h - Edit 1

	LoadICE Version 4
	Copyright (C) 1990-99 Grammar Engine, Inc.
	All rights reserved
	
	NOTICE:  This software source is a licensed copy of Grammar Engine's
	property.  It is supplied to you as part of support and maintenance
	of some Grammar Engine products that you may have purchased.  Use of
	this software is strictly limited to use with such products.  Any
	other use constitutes a violation of this license to you.
*/

/*	 - LoadICE configuration parameters
*/

#define	PIC_ARM	30			/* alarm period is 30 seconds */
#define	PIC_SCR	128			/* max entries in table in pisynf.c */
#define	PIC_NUM 16			/* max numeric items in script */
#define	PIC_LST 16			/* max size of numeric list */
#define	PIC_CS	64			/* command stack size */
#define	PIC_TRY	3			/* reset after this many autobaud chars */
#define	PIC_TOT	3			/* just a second, or two. NOT! */
#define	PIC_NF	20			/* number of files */
#define	PIC_NR	16			/* number of ROMs */
#define	PIC_NC	16			/* number of configurations */
#define	PIC_WS	16			/* max # of ROMs in word */
#define	PIC_KK	24			/* function key count */
#define	PIC_BS	1024		/* bigbuffer size */
#define	PIC_XC	32			/* over the link xfer count */
#define	PIC_SN	32			/* serial device name size */
#define	PIC_PN	32			/* parallel device name size */
#define	PIC_HN	128			/* fastport host name */
#define	PIC_FN	128			/* filename size */
#define	PIC_SS	32			/* number of bytes to search */
#define	PIC_CL	512			/* length of the command line */
#define	PIC_MP	259			/* Max packet size */
#define	PIC_MD	256			/* maximum data in packet */
#define	PIC_DS	63			/* default dump size */
#define	PIC_RT	55			/* reset time default is 55*9 ms */
#define	PIC_TI	48			/* default target intrpt length (*2+4 mics) */
#define	PIC_MM	0x1000000	/* 24-bit address sapce */
#define	PIC_MX	0x10000		/* maximum transfer per unit before pilop() */
#define	PIC_LOCK	-1		/* response to 'mode' if unit is locked */
#define	PIC_XCODE	0x96	/* AITTY mode code */
#define	PIC_DLY		100000	/* loop count before checking for time-out */
#define	PIC_PPD0	10		/* loop count for fast host strobe strech */
#define	PIC_PPD1	10		/* loop count for fast host strobe2 strech */
#define	PIC_PPD2	10		/* loop count for fast host b_ack strech */
#define	PIC_F040	524288	/* 512k ROM */
#ifdef UNIX
#define	PIC_BAUD	19200	/* default baud rate */
#else
#define	PIC_BAUD	57600	/* default baud rate */
#endif
#define	FIC_NS		8		/* number of sectors */
#define PTR_STRIP	18

/*	- and other stuff */

#define	PI_SUCCESS	0			/* mission successfull */
#define	PI_FAILURE	1			/* didn't make it */
