/*	PiSetup.h - Edit 1

	LoadICE Version 4
	Copyright (C) 1990-99 Grammar Engine, Inc.
	All rights reserved
	
	NOTICE:  This software source is a licensed copy of Grammar Engine's
	property.  It is supplied to you as part of support and maintenance
	of some Grammar Engine products that you may have purchased.  Use of
	this software is strictly limited to use with such products.  Any
	other use constitutes a violation of this license to you.
*/

/* Setup data structures and data */

char *pxmm[] = {
	"1. PromICE to Host Connection",
	"2. ROM Configuration",
	"3. File Configuration",
	"4. AI Configuration",
	"5. Misc. Configuration",
	"6. Save Config in LOADICE.INI file",
	"7. Setup Help",
	"8. Exit Setup",
	0
};

char *pxhm[] = {
	"1. Serial - COM Port Only",
	"2. Parallel - LPT Port Only",
	"3. Both - COM & LPT Ports",
	"4. Ethernet - Serial Connection to PromICE",
	"5. Ethernet - Serial & Parallel Connection to PromICE",
	"6. Connect with PromICE Now",
	"7. Exit Port Setup",
	0
};

char *pxgcp[] = {
	"1. COM1",
	"2. COM2",
	"3. COM3",
	"4. COM4",
	0
};

char *pxgpp[] = {
	"1. LPT1",
	"2. LPT2",
	"3. LPT3",
	"4. LPT4",
	0
};

char *pxgbr[] = {
	"1. 1200",
	"2. 2400",
	"3. 4800",
	"4. 9600",
	"5. 19200",
	"6. 57600",
	0
};

char *pxrm[] = {
	"1. Size by Part Number",
	"2. Socket Size",
	"3. Word Size",
	"4. Byte Order",
	"5. Fill Data",
	"6. Exit ROM Setup",
	0
};

char *pxrwz[] = {
	"1.  8 Bits",
	"2. 16 Bits",
	"3. 32 Bits",
	"4. 64 Bits",
	0
};

char *pxrbo[] = {
	"1. Lillte Endian (Motorola like - 0, 1)",
	"2. Big Endian (Intel like - 1, 0)",
	"3. Custom",
	0
};

char *pxru2 = "1 0";
char *pxru4 = "1 0 3 2";
char *pxr32 = "3 2 1 0";
char *pxr64 = "7 6 5 4 3 2 1 0";

char *pxrrz[] = {
	"01. 2716  / 16K  Bits / 2K   Words / 8  Bit Wide",
	"02. 2732  / 32K  Bits / 4K   Words / 8  Bit Wide",
	"03. 2764  / 64K  Bits / 8K   Words / 8  Bit Wide",
	"04. 27128 / 128K Bits / 16K  Words / 8  Bit Wide",
	"05. 27256 / 256K Bits / 32K  Words / 8  Bit Wide",
	"06. 27512 / 512K Bits / 64K  Words / 8  Bit Wide",
	"07. 27010 / 1M   Bits / 128K Words / 8  Bit Wide",
	"08. 27020 / 2M   Bits / 256K Words / 8  Bit Wide",
	"09. 27040 / 4M   Bits / 512K Words / 8  Bit Wide",
	"10. 27080 / 8M   Bits / 1M   Words / 8  Bit Wide",
	"11. 27160 / 16M  Bits / 2M   Words / 8  Bit Wide",
	"12. 2x100 / 1M   Bits / 64K  Words / 16 Bit Wide",
	"13. 2x200 / 2M   Bits / 128K Words / 16 Bit Wide",
	"14. 2x400 / 4M   Bits / 256K Words / 16 Bit Wide",
	"15. 2x800 / 8M   Bits / 512K Words / 16 Bit Wide",
	"16. 2x160 / 16M  Bits / 1M   Words / 16 Bit Wide",
	"17. 2x320 / 32M  Bits / 2M   Words / 16 Bit Wide",
	0
};
char *pxrsz[] = {
	"01. Socket is same as ROM size",
	"02. 27512 / 512K Bits / 64K  Words / 8  Bit Wide",
	"03. 27010 / 1M   Bits / 128K Words / 8  Bit Wide",
	"04. 27020 / 2M   Bits / 256K Words / 8  Bit Wide",
	"05. 27040 / 4M   Bits / 512K Words / 8  Bit Wide",
	"06. 27080 / 8M   Bits / 1M   Words / 8  Bit Wide",
	"07. 27160 / 16M  Bits / 2M   Words / 8  Bit Wide",
	"08. 2x100 / 1M   Bits / 64K  Words / 16 Bit Wide",
	"09. 2x200 / 2M   Bits / 128K Words / 16 Bit Wide",
	"10. 2x400 / 4M   Bits / 256K Words / 16 Bit Wide",
	"11. 2x800 / 8M   Bits / 512K Words / 16 Bit Wide",
	"12. 2x160 / 16M  Bits / 1M   Words / 16 Bit Wide",
	"13. 2x320 / 32M  Bits / 2M   Words / 16 Bit Wide",
	0
};

char *pxrrt[] = {
	"2716","2732","2764","27128","27256","27512",
	"27010","27020","27040","27080","27160",
	"27512","27010","27020","27040","27080","27160",0
};

char *pxrfd[] = {
	"1. Fill the Whole ROM Space with 0xFFs",
	"2. Let me specify parameters",
	0
};

char *pxfty[] = {
	"1. Hex Records of some sort",
	"2. Binary data",
	0
};

char *pxfwz[] = {
	"1. Same as ROM Config",
	"2. Custom Config",
	0
};

char *pxaim[] = {
	"1. Setup AILOC",
	"2. Setup Burst Mode Access",
	"3. Setup AICONTROL Deskew",
	"4. Setup DirectWrite of Target Data (AI2 only)",
	"5. Exit AI Setup",
	0
};

char *pxaisp[] = {
	"1. Serial Port",
	"2. Parallel Port",
	0
};

char *pxabk[] = {
	"1. Don't Break the Link (default)",
	"2. Break on a Single Host Interrupt",
	"3. Specify how many Interrupts to ignore",
	0
};

char *pxaxp[] = {
	"1. Binary - NO Break Character",
	"2. ASCII - Specify a Break Character",
	0
};

char *pxaib1[] = {
	"1. NO Burst Access",
	"2.  4 Byte Burst",
	"3.  8 Byte Burst",
	"4. 16 Byte Burst",
	0
};

char *pxaib2[] = {
	"1. NO Burst Access",
	"2.  2 Byte Burst",
	"3.  4 Byte Burst",
	"4.  8 Byte Burst",
	"5. 16 Byte Burst",
	"6. 32 Byte Burst",
	0
};

char *pxaid1[] = {
	"1. NO Deskew desired",
	"2. 50ns Deskew for Target Data",
	"3. 50ns Deskew for Target & Host Data",
	"4. 50ns write for ROM patching funciton",
	0
};

char *pxaid2[] = {
	"1. Use Default Deskew (best)",
	"2. Use 20ns Deskew",
	"3. Use 40ns Deskew",
	"4. Use 60ns Deskew",
	0
};

char *pxxm[] = {
	"1. Load & Exit",
	"2. Stay in Dialog Mode",
	0
};

char *pxmh[] = {
	"Setup will help you with most common situations",
	" ",
	"You can specify PromICE to Host/PC connection,",
	"ROM size, SOCKET size, WORD size, BYTE ORDER,",
	"FILL character. You can specify FILEs and",
	"Loading Parameters. And if you are using AI",
	"then you can specify AILOC related parameters",
	"You can create a new LOADICE.INI file with these",
	"parameters in it (old one is renamed loadice.old).",
	" ",
	"For more complex operations and additional settings",
	"consult the manual and edit the LOADICE.INI file",
	"accordingly or use dialog mode commands.",
	" ",
	"You can enter setup by a direct command at the",
	"LoadICE: prompt or by running loadice with the",
	"'-setup' argument, and if you don't want loadice",
	"to process current LOADICE.INI file then run",
	"loadice with '. -setup' as the argument",
	" "
	"If you re-enter setup, then make sure you specify",
	"ROM and SOCKET and AI related parameters over again",
	"It will keep your host link parameters and file parameters",
	"But the others must be respecified to be included in new",
	"loadice.ini file",
	0,
};